package udpclient;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */
import java.net.*;
import java.io.*;

/**
 * Diese Klasse gestattet es Ihnen Zeichenketten und / oder
 * Byte Arrays als UDP / Datagramme zu senden und zu empfangen.
 * Sie brauchen sich dabei nicht um die Socket- bzw. Datagram Details
 * zu kmmern.
 */
public class UDPClient {

  InetAddress ia;
  int port;
  DatagramSocket ds;

  /**
   * Kreiere einen neuen UDPClient.
   * @param ia InettAddresse des remote Host (an den die Daten gesandt werden)
   * @param port Port des remote Host an den die Daten gesandt werden
   * @throws SocketException
   */
   public UDPClient(InetAddress ia, int port) throws SocketException {

    this.ia = ia;
    this.port = port;
    ds = new DatagramSocket();

  }

  public UDPClient(String hostname, int port) throws UnknownHostException, SocketException {

    this(InetAddress.getByName(hostname), port);

  }

  /**
   * Senden von Daten an einen remote Host via UDP.
   * Maximale Lnge: 64900) bytes sonst wird eine
   * IOException geworfen
   * @param buffer Byte Array mit den Daten
   * @throws IOException
   */
  public void send(byte[] buffer) throws IOException {

    if (buffer.length > 64900) throw new IOException();
    DatagramPacket dp = new DatagramPacket(buffer, buffer.length, ia, port);
    ds.send(dp);

  }

  /**
   * Senden von ISO-Latin-1 Zeichenketten an den remote Host via UDP
   * Die Zeichenkette wird ISO-Latin-1
   * @param s zu sendende Zeichenkette
   * @throws IOException
   */
  public void send(String s) throws IOException {

    byte[] data = new byte[s.length()];
    s.getBytes(0, s.length(), data, 0);
    send(data);

  }

  /**
   * Senden eines leeren Datagrams via UDP an den remote Host.
   * @throws IOException
   */
  public void send() throws IOException {

    byte[] b = new byte[1];
    send(b);

  }

  /**
   * Warten auf ein Datagram
   * @return the data received as a byte array
   * @throws IOException
   */
  public synchronized byte[] receive() throws IOException {

    byte[] buffer = new byte[65507];
    DatagramPacket incoming = new DatagramPacket(buffer, buffer.length);
    ds.receive(incoming);
    //
    while ( !incoming.getAddress().equals(ia)) {
      ds.receive(incoming);
    }
    return incoming.getData();

  }

  /**
   * Empfangen von Daten vom Host
   * @returnDaten als Byte Array
   * @throws IOException
   */
  public synchronized String receiveString() throws IOException {

    byte[] data = receive();
    return new String(data, 0, 0, data.length);

  }

  /**
   * @return Port an den die Daten gesandt wurden
   */
  public int getPort() {

    return port;

  }

  /**
   * @return Port, an den der Client gebunden ist
   */
  public int getLocalPort() {

    return ds.getLocalPort();

  }

  /**
   * @return InetAddress an die der Client Daten sendet
   */
  public InetAddress getAddress() {

    return ia;

  }

  /**
   * @return Host, an den die Daten gesandt werden
   */
  public String toString() {

    return ia + ":" + port;

  }

}
